//****************************************************************************
//  Copyright ElectricRR - 2013
// This file is part of the OFST controller system.
//
//    OFST is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    OFST is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
//
// MODULE NAME:	command.h
//
// Revision:	1.0
//
// DESCRIPTION: Function prototypes for command.c
//
//*****************************************************************************

#ifndef command_h
#define command_h


/* constants */
#define SERIAL_SEARCH_TIME		250		// 250msec for TMCC status byte

#define TMCC1_STATE				0
#define TMCC2_STATE				1
#define DEBOUNCE_TIME			100		// 100 * 5msec = 500msec


#define STANDALONE_LASHUP 		0
#define FRONT_LASHUP 			1
#define MIDDLE_LASHUP 			2
#define REAR_LASHUP 			3
#define LASHUP_POS_MASK			0x03
#define LASHUP_DIR_MASK			0x04
#define LASHUP_ADDR_MASK		0x0F
#define LASHUP_ADDR2_MASK		0x7F

#define MOMENTUM_LOW 			0
#define MOMENTUM_CONV			1
#define MOMENTUM_MED 			4
#define MOMENTUM_HI 			7

// TMCC 1 commands
#define TMCC1_NOP				0x80
#define TMCC1_HALT				0xAB

#define TMCC1_TOGGLE_DIR		0x01
#define TMCC1_FORWARD			0x00
#define TMCC1_ALT_FORWARD		0x1B
#define TMCC1_REVERSE			0x03

#define TMCC1_BOOST				0x04
#define TMCC1_BRAKE				0x07

#define TMCC1_FRONT_COUPLER		0x05
#define TMCC1_REAR_COUPLER		0x06

#define TMCC1_AUX1_OFF			0x08
#define TMCC1_AUX1				0x09
#define TMCC1_AUX1_ON			0x0B

#define TMCC1_AUX2_OFF			0x0C
#define TMCC1_AUX2				0x0D
#define TMCC1_AUX2_ON			0x0F

#define TMCC1_BELL				0x1D
#define TMCC1_HORN_1			0x1C
#define TMCC1_HORN_2			0x1F

#define TMCC1_CHUFF				0x1E
#define TMCC1_CHUFF_2			0x1A
#define TMCC1_HISS				0x52

#define TMCC1_ZERO				0x10
#define TMCC1_ONE				0x11
#define TMCC1_TWO				0x12
#define TMCC1_THREE				0x13
#define TMCC1_FOUR				0x14
#define TMCC1_FIVE				0x15
#define	TMCC1_SIX				0x16
#define TMCC1_SEVEN				0x17
#define TMCC1_EIGHT				0x18
#define TMCC1_NINE				0x19

#define TMCC1_LASHUP_SGL_FWD  	0x20
#define TMCC1_LASHUP_HEAD_FWD  	0x21
#define TMCC1_LASHUP_MID_FWD  	0x22
#define TMCC1_LASHUP_REAR_FWD  	0x23
#define TMCC1_LASHUP_SGL_REV  	0x24
#define TMCC1_LASHUP_HEAD_REV  	0x25
#define TMCC1_LASHUP_MID_REV  	0x26
#define TMCC1_LASHUP_REAR_REV  	0x27

#define TMCC1_REL_SPEED_MIN		0x40
#define TMCC1_REL_SPEED_BASE	0x45
#define TMCC1_REL_SPEED_MAX		0x4A
#define TMCC1_ABS_SPEED_ZERO	0x60
#define TMCC1_ABS_SPEED_MAX		0x7F

#define TMCC1_LASHUP_CLEAR  	0x2C
#define TMCC1_SET_ADDRESS		0x2B
#define TMCC1_ASSIGN_TRAIN_LO	0x30
#define TMCC1_ASSIGN_TRAIN_HI	0x3F

// TMCC 2 commands
#define TMCC2_SPEED_ZERO		0x00
#define TMCC2_SPEED_MAX			0xC7



/* exports */


/* function prototypes */
void Command_Process();
void Command_Decoder(uchar, uchar);
void TMCC1_Decode(uchar);
void TMCC2_Decode(uchar);
void Multiword_Decode(uchar);
void doMultibyteOperation();

void Ext_Command_Decoder(uint);
void Internal_Multi_Var_Command_Decoder(uint, uchar, uchar *);
void Service_Command_Debounce_Timers();


#endif  // command_h
