//****************************************************************************
//  Copyright ElectricRR - 2013
// This file is part of the OFST controller system.
//
//    OFST is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    OFST is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
//
// MODULE NAME:	eeprom.c
//
// Revision:	1.0
//
// DESCRIPTION: EEprom support routines
//
//*****************************************************************************


/* include files */
#include <htc.h>
#include "main.h"
#include "globals.h"
#include "command.h"
#include "lights.h"
#include "eeprom.h"



//******************************************************************************
//
// FUNCTION NAME:	Init_EEData()
//
// DESCRIPTION:		Initialize data from the EEprom, reset if bad checksum
//
//******************************************************************************

void Init_EEData(void)
{

	if (!ReadPersistantData())
		Reset_All_EEData();
}


//******************************************************************************
//
// FUNCTION NAME:	ReadPersistantData()
//
// DESCRIPTION:		Read and checksums the EEprom data block
//
//******************************************************************************

uchar ReadPersistantData(void)
{
	uint   ix;
	uchar* tmp_ptr;
	uchar  ret_value = FALSE;


	// address of RAM ee_data
	tmp_ptr = (uchar*)&ee_data;

	// move EEprom into RAM ee_data
	for (ix = 0; ix < sizeof(ee_data); ix++)
		tmp_ptr[ix] = EEPROM_READ(ix + EE_DATA_LOCATION);

	// verify RAM ee_data, skipping checksum
	for (ix = 2; ix < (&ee_data.end_marker - tmp_ptr); ix++)
		ee_data.checksum -= tmp_ptr[ix];

	if (ee_data.checksum == CHECKSUM_VALUE)
		ret_value = TRUE;

	return ret_value;
}


//******************************************************************************
//
// FUNCTION NAME:	WritePersistantData()
//
// DESCRIPTION:		Write out the EEprom data block with checksum
//					Use pace_write to slow down updates as needed
//					to prevent flicker on PWM'd lamps.
//
//******************************************************************************

uchar WritePersistantData(uchar pace_write)
{
	uint   ix;
	uchar* tmp_ptr;
	uchar  ret_value = FALSE;


	// address of RAM ee_data
	tmp_ptr = (uchar*)&ee_data;

	// init checksum
	ee_data.checksum = CHECKSUM_VALUE;

	// calc checksum, skipping checksum
	for (ix = 2; ix < (&ee_data.end_marker - tmp_ptr); ix++)
		ee_data.checksum += tmp_ptr[ix];

	// scan and update EEprom when data changes
	for (ix = 0; ix < sizeof(ee_data); ix++)
	{
		if (tmp_ptr[ix] != eeprom_read(ix + EE_DATA_LOCATION))
		{
			eeprom_write(ix + EE_DATA_LOCATION, tmp_ptr[ix]);
			ret_value = TRUE;

			// to pace out writes, used to prevent Lamp PWM flicker
			if (pace_write)
				break;
		}
	}

	return ret_value;
}


//******************************************************************************
//
// FUNCTION NAME:	Reset_All_EEData()
//
// DESCRIPTION:		Default the EEprom persistant data
//
//******************************************************************************

void Reset_All_EEData(void)
{

//TBD: init NV defaults here

	ee_data.ditch_light_flags = DITCH_OFF_PULSE_ON;

	// stash data
	WritePersistantData(0);
}





