//****************************************************************************
//  Copyright ElectricRR - 2013
// This file is part of the OFST controller system.
//
//    OFST is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    OFST is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
//
// MODULE NAME:	globals.c
//
// Revision:	1.0
//
// DESCRIPTION:	Global RAM and general control routines
//
//*****************************************************************************


#include <htc.h>
#include "main.h"
#include "globals.h"
#include "eeprom.h"
#include "command.h"


/* globals */
uchar SerData;					// lower 8 bits of serial data
uchar Com9bit;					// high 9th bit of serial data

struct TCB		tcb_data;		// Train Control Block
struct EEPROM	ee_data;		// EEPROM data


//******************************************************************************
//
// FUNCTION NAME:	Init_TCBData()
//
// DESCRIPTION:		Initialize data from the EEprom, reset if bad checksum
//
//******************************************************************************

void Init_TCBData(void)
{
	uchar  ix;
	uchar* tmp_ptr;

	// address of RAM tcb_data
	tmp_ptr = (uchar*)&tcb_data;

	// init System defaults
	for (ix = 0; ix < sizeof(tcb_data); ix++)
		tmp_ptr[ix] = 0;

	tcb_data.cmnd_mode_detected = TRUE;
}



//**************************************************************************
//
// FUNCTION NAME:	Sample_ADC_Channel()
//
// DESCRIPTION:     return current ADC reading from selected channel
//
//***************************************************************************

uint Sample_ADC_Channel(uchar the_channel)
{

	uchar ix;
	uint  sample = 0xFFFF;

	struct {
		union _address x;
	} adc_reading;


	// protect "busy wait" delays
	CLRWDT();

	// select channel, delay to stabilize (typ = 100usec)
	ADCON0 = (the_channel << 2) | ADON;
	__delay_us(100);

	// start conversion
	ADGO = 1;

	// wait for conversion with timeout (typ = 50 usec)
	for (ix = 0; ix < 10; ix++)
	{
		__delay_us(10);
		
		if (!GO_nDONE)
		{
			adc_reading.x.byte.hi = ADRESH;
			adc_reading.x.byte.lo = ADRESL;
			sample = adc_reading.x.word;
			break;
		}
	}

	return sample;
}




